import * as i0 from '@angular/core';
import { Injectable, SecurityContext, Component, Input, NgModule } from '@angular/core';
import { Subject } from 'rxjs';
import { trigger, state, style, transition, animate } from '@angular/animations';
import { takeUntil } from 'rxjs/operators';
import * as i2 from '@angular/platform-browser';
import * as i3 from '@angular/common';
import { CommonModule } from '@angular/common';

/** Notification Class */
class Notification {
    /**
     * Constructor For Notification
     *
     * @param message message to be shown
     * @param style bootstrap background colors
     * @param duration timeout for notification
     */
    constructor(message, style, duration) {
        this.id = `_${Math.random().toString(36).substring(2, 11)}`; // generate a random number
        this.message = message;
        this.style = style || 'info';
        this.duration = duration;
    }
}

/** Notification Service, recieves notifications from user and interacts with components */
class NgxNotifierService {
    constructor() {
        /** notification which can be subscribed on new messages */
        this.notification = new Subject();
        /** clear all toast notifications */
        this.clearToasts = new Subject();
        /** clear last toast notification */
        this.clearLastToast = new Subject();
    }
    /** pushes a new notification */
    createToast(message, style, duration) {
        this.notification.next(new Notification(message, style, duration));
    }
    /** clear all toast notifications */
    clear() {
        this.clearToasts.next();
    }
    /** clear last toast notification */
    clearLast() {
        this.clearLastToast.next();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.2.3", ngImport: i0, type: NgxNotifierService, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "17.2.3", ngImport: i0, type: NgxNotifierService }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.2.3", ngImport: i0, type: NgxNotifierService, decorators: [{
            type: Injectable
        }] });

/**
 * Notifier compoent, which holds all the notifications can be accessed via `ngx-notifier` selector
 */
class NgxNotifierComponent {
    /**
     * NgxNotifierComponent Constructor
     *
     * @param ngxNotifierService subscribe to get values form notifier service
     */
    constructor(ngxNotifierService, domSanitizer) {
        this.ngxNotifierService = ngxNotifierService;
        this.domSanitizer = domSanitizer;
        this.componentDestroyed$ = new Subject();
        /** whether to allow duplicate messages or not */
        this.allowDuplicates = true;
        /** allow HTML in notification */
        this.allowHTML = false;
        /** custom class to be attached */
        this.className = '';
        /** default duration for dismissing notifications (60s/1minute) */
        this.duration = 60000;
        /** weather to enable or disable animations */
        this.disableAnimations = false;
        /** click to dismiss a notification */
        this.dismissOnClick = false;
        /** whether to insert on top or at bottom */
        this.insertOnTop = true;
        /** Maximum number of notifications to keep */
        this.max = 5;
        /** array of notifications */
        this.notifications = [];
        this.ngxNotifierService.notification.pipe(takeUntil(this.componentDestroyed$)).subscribe((notification) => { this.updateNotifications(notification); });
        this.ngxNotifierService.clearToasts.pipe(takeUntil(this.componentDestroyed$)).subscribe(() => { this.notifications = []; });
        this.ngxNotifierService.clearLastToast.pipe(takeUntil(this.componentDestroyed$)).subscribe(() => { this.clearLastToast(); });
    }
    /**
     * updates notification into the array i.e., Which is the display
     *
     * @param notification notification element
     */
    updateNotifications(notification) {
        // checks whether the message is alrady present in notifications
        const index = this.notifications.map((e) => e.message).indexOf(notification.message);
        if (!this.allowDuplicates) {
            if (index !== -1) {
                return;
            }
        }
        // save the last inserted Id
        this.lastInsertedNotificationId = notification.id;
        // sanitize html if enableHTML is set to true
        let sanitizedMessage = null;
        if (notification.message && this.allowHTML) {
            sanitizedMessage = this.domSanitizer.sanitize(SecurityContext.HTML, notification.message);
        }
        // set sanitized output to notification message
        notification.message = sanitizedMessage || notification.message;
        // insert notification in the first position of the array
        if (this.insertOnTop) {
            this.notifications.unshift(notification);
        }
        else {
            this.notifications.push(notification);
        }
        /**
         * remove the last inserted element if max has
         * pop or shift based on `insertOnTop`
         */
        if (this.notifications.length > this.max) {
            if (this.insertOnTop) {
                this.notifications.pop();
            }
            else {
                this.notifications.shift();
            }
        }
        // clear notification in given time
        setTimeout(() => {
            this.notifications.splice(index, 1);
        }, notification.duration || this.duration);
    }
    /**
     * remove the element from the array based on index
     *
     * @param index position of the element
     */
    removeNotification(index) {
        if (index !== undefined || index !== null) {
            this.notifications.splice(index, 1);
        }
    }
    /**
     * click event when toast is cleared
     *
     * @param index position of the element
     */
    onToastClick(index) {
        if (this.dismissOnClick) {
            this.removeNotification(index);
        }
    }
    // dummy keyup handler
    onKeyUp() {
        // do nothing
    }
    /** clear last inserted toast notification */
    clearLastToast() {
        const index = this.notifications.map((e) => e.id).indexOf(this.lastInsertedNotificationId);
        if (this.notifications.length !== 0 && index !== -1) {
            this.notifications.splice(index, 1);
        }
    }
    /** stop subscription when component is destroyed */
    ngOnDestroy() {
        this.componentDestroyed$.next(true);
        this.componentDestroyed$.complete();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.2.3", ngImport: i0, type: NgxNotifierComponent, deps: [{ token: NgxNotifierService }, { token: i2.DomSanitizer }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "17.2.3", type: NgxNotifierComponent, selector: "ngx-notifier", inputs: { allowDuplicates: "allowDuplicates", allowHTML: "allowHTML", className: "className", duration: "duration", disableAnimations: "disableAnimations", dismissOnClick: "dismissOnClick", insertOnTop: "insertOnTop", max: "max" }, ngImport: i0, template: "<div\n  class=\"ngx-notifier\"\n  *ngIf=\"notifications?.length !== 0\"\n  [ngClass]=\"className\"\n  [@animateToasts]\n  [@.disabled]=\"disableAnimations\"\n>\n  <div [ngClass]=\"{ pointer: dismissOnClick }\">\n    <div\n      class=\"ngx-n-notification alert\"\n      role=\"alert\"\n      *ngFor=\"let notification of notifications; let i = index\"\n      [ngClass]=\"'alert-' + notification.style\"\n      (click)=\"onToastClick(i)\"\n      (keyup)=\"onKeyUp()\"\n      [@animateToasts]\n    >\n      <button type=\"button\" class=\"pointer close\" aria-label=\"Close\" (click)=\"removeNotification(i)\">\n        <svg\n          xmlns=\"http://www.w3.org/2000/svg\"\n          width=\"12\"\n          height=\"12\"\n          fill=\"currentColor\"\n          viewBox=\"0 0 16 16\"\n          aria-hidden=\"true\"\n        >\n          <path\n            d=\"M1.293 1.293a1 1 0 0 1 1.414 0L8 6.586l5.293-5.293a1 1 0 1 1 1.414 1.414L9.414 8l5.293 5.293a1 1 0 0 1-1.414 1.414L8 9.414l-5.293 5.293a1 1 0 0 1-1.414-1.414L6.586 8 1.293 2.707a1 1 0 0 1 0-1.414z\"\n          />\n        </svg>\n      </button>\n      <p *ngIf=\"allowHTML\" [innerHTML]=\"notification.message\"></p>\n      <p *ngIf=\"!allowHTML\">{{ notification.message }}</p>\n    </div>\n  </div>\n</div>\n", styles: ["*,*:after,*:before{box-sizing:border-box}.ngx-notifier{position:fixed;min-width:25rem;max-width:25rem;right:1rem;bottom:1rem}.ngx-notifier .pointer{cursor:pointer}.ngx-notifier .ngx-n-notification{margin-top:.5rem;border-radius:4px;padding:.5rem}.ngx-notifier .ngx-n-notification p{margin:0}.ngx-notifier .ngx-n-notification .close:focus{outline:none}.ngx-notifier .alert{position:relative;padding:1rem 3rem 1rem 1rem;margin-bottom:1rem;border:1px solid transparent;border-radius:.25rem}.ngx-notifier .alert-success{color:#0f5132;background-color:#d1e7dd;border-color:#badbcc}.ngx-notifier .alert-info{color:#055160;background-color:#cff4fc;border-color:#b6effb}.ngx-notifier .alert-warning{color:#664d03;background-color:#fff3cd;border-color:#ffecb5}.ngx-notifier .alert-danger{color:#842029;background-color:#f8d7da;border-color:#f5c2c7}.ngx-notifier .close{position:absolute;top:0;right:0;z-index:2;padding:1.25rem 1rem;width:1rem;height:1rem;background-color:transparent;border:none;box-sizing:content-box}@media only screen and (max-width: 420px){.ngx-notifier{min-width:20rem;max-width:20rem}}\n"], dependencies: [{ kind: "directive", type: i3.NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { kind: "directive", type: i3.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i3.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }], animations: [
            trigger('animateToasts', [
                state('void', style({ opacity: 0 })),
                transition(':enter, :leave', [
                    animate('0.3s ease'),
                ]),
            ]),
        ] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.2.3", ngImport: i0, type: NgxNotifierComponent, decorators: [{
            type: Component,
            args: [{ selector: 'ngx-notifier', animations: [
                        trigger('animateToasts', [
                            state('void', style({ opacity: 0 })),
                            transition(':enter, :leave', [
                                animate('0.3s ease'),
                            ]),
                        ]),
                    ], template: "<div\n  class=\"ngx-notifier\"\n  *ngIf=\"notifications?.length !== 0\"\n  [ngClass]=\"className\"\n  [@animateToasts]\n  [@.disabled]=\"disableAnimations\"\n>\n  <div [ngClass]=\"{ pointer: dismissOnClick }\">\n    <div\n      class=\"ngx-n-notification alert\"\n      role=\"alert\"\n      *ngFor=\"let notification of notifications; let i = index\"\n      [ngClass]=\"'alert-' + notification.style\"\n      (click)=\"onToastClick(i)\"\n      (keyup)=\"onKeyUp()\"\n      [@animateToasts]\n    >\n      <button type=\"button\" class=\"pointer close\" aria-label=\"Close\" (click)=\"removeNotification(i)\">\n        <svg\n          xmlns=\"http://www.w3.org/2000/svg\"\n          width=\"12\"\n          height=\"12\"\n          fill=\"currentColor\"\n          viewBox=\"0 0 16 16\"\n          aria-hidden=\"true\"\n        >\n          <path\n            d=\"M1.293 1.293a1 1 0 0 1 1.414 0L8 6.586l5.293-5.293a1 1 0 1 1 1.414 1.414L9.414 8l5.293 5.293a1 1 0 0 1-1.414 1.414L8 9.414l-5.293 5.293a1 1 0 0 1-1.414-1.414L6.586 8 1.293 2.707a1 1 0 0 1 0-1.414z\"\n          />\n        </svg>\n      </button>\n      <p *ngIf=\"allowHTML\" [innerHTML]=\"notification.message\"></p>\n      <p *ngIf=\"!allowHTML\">{{ notification.message }}</p>\n    </div>\n  </div>\n</div>\n", styles: ["*,*:after,*:before{box-sizing:border-box}.ngx-notifier{position:fixed;min-width:25rem;max-width:25rem;right:1rem;bottom:1rem}.ngx-notifier .pointer{cursor:pointer}.ngx-notifier .ngx-n-notification{margin-top:.5rem;border-radius:4px;padding:.5rem}.ngx-notifier .ngx-n-notification p{margin:0}.ngx-notifier .ngx-n-notification .close:focus{outline:none}.ngx-notifier .alert{position:relative;padding:1rem 3rem 1rem 1rem;margin-bottom:1rem;border:1px solid transparent;border-radius:.25rem}.ngx-notifier .alert-success{color:#0f5132;background-color:#d1e7dd;border-color:#badbcc}.ngx-notifier .alert-info{color:#055160;background-color:#cff4fc;border-color:#b6effb}.ngx-notifier .alert-warning{color:#664d03;background-color:#fff3cd;border-color:#ffecb5}.ngx-notifier .alert-danger{color:#842029;background-color:#f8d7da;border-color:#f5c2c7}.ngx-notifier .close{position:absolute;top:0;right:0;z-index:2;padding:1.25rem 1rem;width:1rem;height:1rem;background-color:transparent;border:none;box-sizing:content-box}@media only screen and (max-width: 420px){.ngx-notifier{min-width:20rem;max-width:20rem}}\n"] }]
        }], ctorParameters: () => [{ type: NgxNotifierService }, { type: i2.DomSanitizer }], propDecorators: { allowDuplicates: [{
                type: Input
            }], allowHTML: [{
                type: Input
            }], className: [{
                type: Input
            }], duration: [{
                type: Input
            }], disableAnimations: [{
                type: Input
            }], dismissOnClick: [{
                type: Input
            }], insertOnTop: [{
                type: Input
            }], max: [{
                type: Input
            }] } });

class NgxNotifierModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.2.3", ngImport: i0, type: NgxNotifierModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "17.2.3", ngImport: i0, type: NgxNotifierModule, declarations: [NgxNotifierComponent], imports: [CommonModule], exports: [NgxNotifierComponent] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "17.2.3", ngImport: i0, type: NgxNotifierModule, providers: [NgxNotifierService], imports: [CommonModule] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.2.3", ngImport: i0, type: NgxNotifierModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CommonModule],
                    declarations: [NgxNotifierComponent],
                    providers: [NgxNotifierService],
                    exports: [NgxNotifierComponent],
                }]
        }] });

/*
 * Public API Surface of ngx-notifier
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgxNotifierComponent, NgxNotifierModule, NgxNotifierService };
//# sourceMappingURL=ngx-notifier.mjs.map
